//=========================================================
//  MusE
//  Linux Music Editor
//    $Id: mixdowndialog.cpp,v 1.2 2002/02/13 11:42:56 muse Exp $
//  (C) Copyright 2001 Werner Schweer (ws@seh.de)
//=========================================================

#include "mixdowndialog.h"
#include "globals.h"
#include <qtoolbutton.h>
#include <qpushbutton.h>
#include <qfiledialog.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include "../sf/sndfile.h"

//---------------------------------------------------------
//   sndFileOpen
//    sf - old soundfile, used to preset file parameters
//---------------------------------------------------------

SndFile* getSndFile(const SndFile* sf, QWidget* parent, const char* name)
      {
      MixdownFileDialog* dialog = new MixdownFileDialog(sf, parent,
         name, true);
      dialog->exec();
      SndFile* sndFile = dialog->sndFile();
      delete dialog;
      return sndFile;
      }

//---------------------------------------------------------
//   MixdownFileDialog
//---------------------------------------------------------

MixdownFileDialog::MixdownFileDialog(const SndFile* _sf,
   QWidget* parent = 0, const char* name = 0, bool modal = false, WFlags fl = 0)
   : MixdownFileDialogBase(parent, name, true, fl)
      {
      sf   = 0;
      connect(buttonPath, SIGNAL(clicked()), SLOT(fdialog()));
      if (_sf) {
            int channels = _sf->channels();
            int format   = _sf->format();
            int bits     = _sf->sampleBits();
            switch(bits) {
                  case 16: format = 0; break;
                  case 24: format = 1; break;
                  case 32: format = 2; break;
                  }
            switch(channels) {
                  case 1:  channels = 1; break;
                  case 2:  channels = 0; break;
                  case 6:  channels = 2; break;
                  }
            editPath->setText(_sf->path());
            comboChannel->setCurrentItem(channels);
            comboFormat->setCurrentItem(format);
            }
      }

//---------------------------------------------------------
//   accept
//---------------------------------------------------------

void MixdownFileDialog::accept()
      {
printf("accept1\n");
      QString oldpath;
      unsigned channel = comboChannel->currentItem();
      unsigned format  = comboFormat->currentItem();
      int bits = 16;
      switch (channel) {
            case 0: channel = 2; break;
            case 1: channel = 1; break;
            case 2: channel = 6; break;     // not implemented!
            }
      switch (format) {
            case 0:     // 16 bit wave
                  format = SF_FORMAT_WAV | SF_FORMAT_PCM;
                  bits   = 16;
                  break;
            case 1:     // 24 bit wave
                  format = SF_FORMAT_WAV | SF_FORMAT_PCM;
                  bits   = 24;
                  break;
            case 2:     // 32 bit float wave
                  format = SF_FORMAT_WAV | SF_FORMAT_FLOAT;
                  bits   = 32;
                  break;
            }
      QString path = editPath->text();
      if (path.isEmpty()) {
            sf = 0;
            reject();
            return;
            }
      if (path.right(4) != ".wav")
            path += ".wav";
      sf = new SndFile(path);
      sf->setFormat(format, channel, sampleRate, bits);
      done(1);
      }

//---------------------------------------------------------
//   fdialog
//---------------------------------------------------------

void MixdownFileDialog::fdialog()
      {
      QString oldpath;
      if (sf)
            oldpath = sf->path();
      QString path = QFileDialog::getSaveFileName(
         oldpath, "Wave Files (*.wav);;All Files (*)", this, "MixdownFileDialog");
      if (!path.isEmpty())
            editPath->setText(path);
      }

